<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
/*
 * Lib file for wcln progress block.
 *
 * @package   block_wcln_progress
 * @copyright 2020 WCLN
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

abstract class wcln_progress {

    /**
     * @const string The page type for this block.
     */
    const PAGE_TYPE = 'block-wcln_progress';

    public static function _s($key, $a = null) {
        return get_string($key, 'block_wcln_progress', $a);
    }

    private static function format_time($time) {
        return userdate($time, '%A, %d %B %Y, %I:%M %P');
    }

    public static function cleanup($table, $contextid, $itemid) {
        global $DB;

        // Clean up the files associated with this email
        // Fortunately, they are only db references, but
        // they shouldn't be there, nonetheless.
        $tablename = explode('_', $table);
        $filearea = end($tablename);

        $fs = get_file_storage();

        $fs->delete_area_files(
            $contextid, 'block_wcln_progress',
            'attachment_' . $filearea, $itemid
        );

        $fs->delete_area_files(
            $contextid, 'block_wcln_progress',
            $filearea, $itemid
        );

        return $DB->delete_records($table, array('id' => $itemid));
    }

    public static function history_cleanup($contextid, $itemid) {
        return self::cleanup('block_wcln_progress_log', $contextid, $itemid);
    }

    public static function draft_cleanup($contextid, $itemid) {
        return self::cleanup('block_wcln_progress_drafts', $contextid, $itemid);
    }

    public static function flatten_subdirs($tree, $genlink, $level=0) {
        $attachments = $spaces = '';
        $spaces = '';
        foreach (range(0, $level) as $space) {
            $spaces .= " - ";
        }
        foreach ($tree['files'] as $filename => $file) {
            $attachments .= $spaces . " " . $genlink($filename) . "\n<br/>";
        }
        foreach ($tree['subdirs'] as $dirname => $subdir) {
            $attachments .= $spaces . " ". $dirname . "\n<br/>";
            $attachments .= self::flatten_subdirs($subdir, $genlink, $level + 2);
        }

        return $attachments;
    }
    public static function filter_roles($userroles, $masterroles) {
        return array_uintersect($masterroles, $userroles, function($a, $b) {
            return strcmp($a->shortname, $b->shortname);
        });
    }

    private static function load_config($courseid) {
        global $DB;

        $fields = 'name,value';
        $params = array('coursesid' => $courseid);
        $table = 'block_wcln_progress_config';

        $config = $DB->get_records_menu($table, $params, '', $fields);

        if (empty($config)) {
            $m = 'moodle';
            $allowstudents = get_config($m, 'block_wcln_progress_allowstudents');
            $roleselection = get_config($m, 'block_wcln_progress_roleselection');
            $prepender = get_config($m, 'block_wcln_progress_prepend_class');
            $receipt = get_config($m, 'block_wcln_progress_receipt');
            $ferpa = get_config($m, 'block_wcln_progress_ferpa');
            // Convert Never (-1) to No (0) in case site config is changed.
            if ($allowstudents == -1) {
                $allowstudents = 0;
            }
            $config = array(
                'allowstudents' => $allowstudents,
                'roleselection' => $roleselection,
                'prepend_class' => $prepender,
                'receipt' => $receipt,
                'ferpa' => $ferpa
            );
        } else {
            // See if allow students is disabled at the site level.
            $allowstudents = get_config('moodle', 'block_wcln_progress_allowstudents');
            if ($allowstudents == -1) {
                $config['allowstudents'] = 0;
            }
            $config['ferpa'] = get_config('moodle', 'block_wcln_progress_ferpa');
        }

        return $config;
    }

    private static function default_config($courseid) {
        global $DB;
        $params = array('coursesid' => $courseid);
        $DB->delete_records('block_wcln_progress_config', $params);
    }

    private static function save_config($courseid, $data) {
        global $DB;
        self::default_config($courseid);

        foreach ($data as $name => $value) {
            $config = new stdClass;
            $config->coursesid = $courseid;
            $config->name = $name;
            $config->value = $value;
            $DB->insert_record('block_wcln_progress_config', $config);
        }
    }

    public static function delete_dialog($courseid, $type, $typeid) {
        global $CFG, $DB, $USER, $OUTPUT;

        $email = $DB->get_record('block_wcln_progress_'.$type, array('id' => $typeid));

        if (empty($email)) {
            print_error('not_valid_typeid', 'block_wcln_progress', '', $typeid);
        }
        $params = array('courseid' => $courseid, 'type' => $type);
        $yesparams = $params + array('typeid' => $typeid, 'action' => 'confirm');

        $optionyes = new moodle_url('/blocks/wcln_progress/emaillog.php', $yesparams);
        $optionno = new moodle_url('/blocks/wcln_progress/emaillog.php', $params);

        $table = new html_table();
        $table->head = array(get_string('date'), self::_s('subject'));
        if ($courseid == 1 ) {
            $table->data = array(
                new html_table_row(array(
                new html_table_cell(self::format_time($email->time)),
                new html_table_cell($email->subject))
             )
             );
        } else {
            $table->data = array(
                new html_table_row(array(
                new html_table_cell(self::format_time($email->time)),
                new html_table_cell($email->subject))
                )
             );
        }
        $msg = self::_s('delete_confirm', html_writer::table($table));

        $html = $OUTPUT->confirm($msg, $optionyes, $optionno);
        return $html;
    }

    public static function list_entries($courseid, $type, $page, $perpage, $userid, $count, $candelete) {
        global $CFG, $DB, $OUTPUT;

        $dbtable = 'block_wcln_progress_'.$type;

        $table = new html_table();

        $params = array('courseid' => $courseid, 'userid' => $userid);
        $logs = $DB->get_records($dbtable, $params,
            'time DESC', '*', $page * $perpage, $perpage);
        if (empty($config['messenger'])) {
            $msg = '0';
        } else {
            $msg = $config['messenger'];
        }

        if ($courseid == '1') {
            $table->head = array(get_string('date'), self::_s('subject'), get_string('action'),
             self::_s('status'), self::_s('failed_to_send_to'), self::_s('send_again'));
        } else {
            if ($msg == 1 ) {
                $table->head = array(get_string('date'), self::_s('subject'),
                         get_string('action'), self::_s('status'),
                         self::_s('failed_to_send_to'), self::_s('send_again'));
            } else {
                $table->head = array(get_string('date'), get_string('action'), self::_s('status'),
                                self::_s('failed_to_send_to'), self::_s('send_again'));
            }
        }
        if (empty($config['messenger'])) {
            $msg = '0';
        } else {
            $msg = $config['messenger'];
        }
        $table->data = array();
        foreach ($logs as $log) {
            $arrayoffaileduserids = array();
            $date = date('Y-m-d H:i:s', $log->time);
            if ($msg == 1) {
                $subject = $log->subject;
            }
            if (!empty($log->failuserids)) {
                $arrayoffaileduserids = explode(",", $log->failuserids);
            }
            $params = array(
                'courseid' => $log->courseid,
                'type' => $type,
                'typeid' => $log->id,
                'groups' => 0
            );

            $actions = array();
            if ($courseid == '1') {
                $openlink = html_writer::link(
                new moodle_url('/blocks/wcln_progress/admin_email.php', $params),
                $OUTPUT->pix_icon('i/search', 'Open Email')
                );
            } else {
                $openlink = html_writer::link(
                    new moodle_url('/blocks/wcln_progress/email.php', $params),
                        $OUTPUT->pix_icon('i/search', 'Open Email')
                );
            }
            $actions[] = $openlink;

            if ($candelete) {
                $deleteparams = $params + array(
                    'userid' => $userid,
                    'action' => 'delete'
                );

                $deletelink = html_writer::link (
                    new moodle_url('/blocks/wcln_progress/emaillog.php', $deleteparams),
                    $OUTPUT->pix_icon("i/invalid", "Delete Email")
                );

                $actions[] = $deletelink;
            }
            $actionlinks = implode(' ', $actions);
            $statussentornot = self::_s($type."success");

            if (!empty($arrayoffaileduserids)) {
                $statussentornot = self::_s('message_failure');
                $params += array(
                    'fmid' => 1,
                );
                $text = self::_s('send_again');
                if ($courseid == '1') {
                    $sendagain = html_writer::link(new moodle_url("/blocks/wcln_progress/admin_email.php", $params), $text);
                } else {
                    $sendagain = html_writer::link(new moodle_url("/blocks/wcln_progress/email.php", $params), $text);
                }
                $listfailids = count($arrayoffaileduserids);
                $failcount = (($listfailids === 1) ? $listfailids . " " .
                 self::_s("user") : $listfailids . " " . self::_s("users"));

            } else {
                $listfailids = $arrayoffaileduserids;
                $sendagain = "";
                $failcount = "";
            }
            if ($courseid == 1) {
                $table->data[] = array($date, $subject, $actionlinks, $statussentornot, $failcount, $sendagain);
            } else {
                if ($msg == 1 ) {
                    $table->data[] = array($date, $subject,  $actionlinks, $statussentornot, $failcount, $sendagain);
                } else {
                    $table->data[] = array($date, $actionlinks, $statussentornot, $failcount, $sendagain);
                }
            }
        }
        $paging = $OUTPUT->paging_bar($count, $page, $perpage,
            '/blocks/wcln_progress/emaillog.php?type='.$type.'&amp;courseid='.$courseid.'&userid='.$userid);

        $html = $paging;
        $html .= html_writer::table($table);
        $html .= $paging;
        return $html;
    }

    /**
     * get all users for a given context
     * @param $context a moodle context id
     * @return array of sparse user objects
     */
    public static function get_all_users($context) {
        global $DB, $CFG;
        // List everyone with role in course.
        //
        // Note that users with multiple roles will be squashed into one
        // record.
        $getnamestring = 'u.firstname, u.lastname';

        if ($CFG->version >= 2013111800) {
        	   $getnamestring = 'firstname, lastname';
        }
        $sturole = $DB->get_field('role', 'id', array('shortname' => 'student'));
        $sql = "SELECT DISTINCT u.id, " . $getnamestring . ",
                u.email, u.mailformat, u.suspended, u.maildisplay
                FROM {role_assignments} ra
                JOIN {user} u ON u.id = ra.userid
                JOIN {role} r ON ra.roleid = r.id
                WHERE (ra.contextid = ? and ra.roleid = ".$sturole.") ";
        $everyone = $DB->get_records_sql($sql, array($context->id));

        return $everyone;
    }

    /**
     * @TODO this function relies on self::get_all_users, it should not have to
     *
     * returns all users enrolled in a gived coure EXCEPT for those whose
     * mdl_user_enrolments.status field is 1 (suspended)
     * @param $context  moodle context id
     * @param $courseid the course id
     */
    public static function get_non_suspended_users($context, $courseid) {
        global $DB, $CFG;
        $everyone = self::get_all_users($context);
        $getnamestring = 'u.firstname, u.lastname';
        if ($CFG->version >= 2013111800) {
            $getnamestring = 'firstname, lastname';
        }
        $sql = "SELECT u.id, " . $getnamestring . " , u.email, u.mailformat, u.suspended, u.maildisplay, ue.status
            FROM {user} u
                JOIN {user_enrolments} ue
                    ON u.id = ue.userid
                JOIN {enrol} en
                    ON en.id = ue.enrolid
                WHERE en.courseid = ?
                    AND ue.status = ?
                ORDER BY u.lastname, u.firstname";
        // Let's use a recordset in case the enrollment is huge.
        $rsvalids = $DB->get_recordset_sql($sql, array($courseid, 0));
        // Container for user_enrolments records.
        $valids = array();
        foreach ($rsvalids as $rsv) {
            $valids[$rsv->id] = $rsv;
        }
        // Required to close the recordset.
        $rsvalids->close();

        // Get the intersection of self::all_users and this potentially shorter list.
        $evryonenotsuspended = array_intersect_key($valids, $everyone);

        return $evryonenotsuspended;
    }

    public static function clean($failuserids) {
        $additionalemails = array();
        $failuserids = explode(',', $failuserids);

        foreach ($failuserids as $id => $failedaddressorid) {
            if ( ! is_numeric($failedaddressorid)) {
                $additionalemails [] = $failedaddressorid;
                 unset($failuserids[$id]);
            }
        }

        $additionalemails = implode(',', $additionalemails);
        $mailto            = implode(',', $failuserids);

        return array($mailto, $additionalemails);
    }
}

function block_wcln_progress_pluginfile($course, $record, $context, $filearea, $args, $forcedownload) {
    $fs = get_file_storage();
    global $DB, $CFG;

    if (!empty($CFG->block_wcln_progress_downloads)) {

        require_course_login($course, true, $record);
    }

    list($itemid, $filename) = $args;

    if ($filearea == 'attachment_log') {
        $time = $DB->get_field('block_wcln_progress_log', 'time', array(
            'id' => $itemid
        ));

        if ("{$time}_attachments.zip" == $filename) {
            $path = wcln_progress::zip_attachments($context, 'log', $itemid);
            send_temp_file($path, 'attachments.zip');
        }
    }

    $params = array(
        'component' => 'block_wcln_progress',
        'filearea' => $filearea,
        'itemid' => $itemid,
        'filename' => $filename
    );

    $instanceid = $DB->get_field('files', 'id', $params);

    if (empty($instanceid)) {
        send_file_not_found();
    } else {
        $file = $fs->get_file_by_id($instanceid);
        send_stored_file($file);
    }
}
